/**
 *   Copyright (c) Rich Hickey. All rights reserved.
 *   The use and distribution terms for this software are covered by the
 *   Eclipse Public License 1.0 (http://opensource.org/licenses/eclipse-1.0.php)
 *   which can be found in the file epl-v10.html at the root of this distribution.
 *   By using this software in any fashion, you are agreeing to be bound by
 * 	 the terms of this license.
 *   You must not remove this notice, or any other, from this software.
 **/

package com.trifork.clj_ds;


public class MapEntry<K,V> extends AMapEntry<K,V>{
final K _key;
final V _val;

public MapEntry(K key, V val){
	this._key = key;
	this._val = val;
}

public K key(){
	return _key;
}

public V val(){
	return _val;
}

public K getKey(){
	return key();
}

public V getValue(){
	return val();
}


}
